require "spec_helper"

describe "Postjob.enqueue!" do
  let(:workflow_name) { "WorkflowName" }
  before(:all) do
    Postjob.register_workflow "WorkflowName"
  end

  include TestHelper

  context "with valid arguments" do
    it "creates a job" do
      Postjob.enqueue! workflow_name
      expect(TestHelper.jobs_count).to eq(1)
    end

    it "returns the created job" do
      id = Postjob.enqueue! workflow_name
      newest_job = TestHelper.newest_job
      expect(newest_job.id).to eq(id)
    end

    it "uses the parent id and sets a full id with two jobs" do
      id1 = Postjob.enqueue! workflow_name
      id2 = Postjob.enqueue! workflow_name, parent_id: id1

      job1 = load_job id1
      job2 = load_job id2

      expect(job2.full_id).to eq("#{job1.id}.#{job2.id}")
    end

    it "uses the parent id and sets a full id" do
      id1 = Postjob.enqueue! workflow_name
      id2 = Postjob.enqueue! workflow_name, parent_id: id1
      id3 = Postjob.enqueue! workflow_name, parent_id: id2

      job1 = load_job id1
      job2 = load_job id2
      job3 = load_job id3

      expect(job3.full_id).to eq("#{job1.id}.#{job2.id}.#{job3.id}")
      expect(job3.full_id).to match(/\d+\.\d+\.\d+/)
    end

    it "sets arguments" do
      id1 = Postjob.enqueue! workflow_name, 1, "two", "three"
      job1 = load_job id1
      expect(job1.args).to eq([1, "two", "three"])

      id2 = Postjob.enqueue! workflow_name
      job2 = load_job id2
      expect(job2.args).to eq([])
    end

    it "sets the queue to the default queue name ('ruby')" do
      id1 = Postjob.enqueue! workflow_name
      job1 = load_job id1
      expect(job1.queue).to eq(Postjob::DEFAULT_QUEUE)

      id1 = Postjob.enqueue! workflow_name, queue: "bla"
      job1 = load_job id1
      expect(job1.queue).to eq("bla")
    end

    it "sets max_attempts" do
      id1 = Postjob.enqueue! workflow_name
      job1 = load_job id1
      expect(job1.max_attempts).to eq(5)

      id1 = Postjob.enqueue! workflow_name, max_attempts: 2
      job1 = load_job id1
      expect(job1.max_attempts).to eq(2)
    end

    it "sets the status to ready" do
      id1 = Postjob.enqueue! workflow_name, 1, "two", "three"
      job1 = load_job id1
      expect(job1.status).to eq("ready")
    end
  end

  context "with invalid arguments" do
    it "raises an error with missing args" do
      expect { Postjob.enqueue! queue: "ruby" }.to raise_error(ArgumentError)
      expect { Postjob.enqueue! workflow: "ruby" }.to raise_error(ArgumentError)
    end

    it "raises an error with invalid args" do
      expect { Postjob.enqueue! queue: 1, workflow: 13 }.to raise_error(ArgumentError)
    end

    it "raises an error with invalid parent_id" do
      id1 = Postjob.enqueue! workflow_name

      expect { Postjob.enqueue! workflow_name, parent_id: id1 - 1 }.to raise_error(PG::ForeignKeyViolation)
    end
  end
end
