require "spec_helper"

module SearchWorkflow
  def self.run
    job = async :manual, timeout: 10

    _token = workflow_token job

    manual_result = await job

    "manual-result:#{manual_result}"
  end

  Postjob.register_workflow self
end

describe "Postjob::Queue.search" do
  include TestHelper

  before do
    Simple::SQL.ask("DELETE FROM postjob.postjobs")
  end

  let!(:id) { Postjob.enqueue! "SearchWorkflow", tags: { "initial" => "yay", "multi" => "manyay" } }

  Queue = Postjob::Queue

  context "attributes" do
    it "returns a specific set of attributes" do
      scope = Postjob::Queue.search("postjobs", id: id)
      result = Simple::SQL.ask scope, into: Hash

      expected_keys = [
        :id,
        :full_id,
        :job,
        :status,
        :error,
        :result,
        :next_run_at,
        :error_backtrace,
        :age,
        :runtime,
        :tags
      ]
      expect(result.keys).to include(*expected_keys)
    end

    it "with attributes: argument it only returns the specified attributes" do
      scope = Postjob::Queue.search("postjobs", attributes: %w(id))
      result = Simple::SQL.ask scope, into: Hash

      expected_keys = [
        :id
      ]
      expect(result.keys).to contain_exactly(*expected_keys)
    end
  end

  context "with multiple jobs" do
    before do
      Postjob.enqueue! "SearchWorkflow", tags: { "secondary" => "yay" }
      expect(Simple::SQL.ask("SELECT COUNT(*) FROM postjob.postjobs")).to eq(2)
    end

    it "returns all entries" do
      scope = Postjob::Queue.search("postjobs")
      result = Simple::SQL.all(scope, into: OpenStruct)
      expect(result.length).to eq(2)
    end
  end

  describe "filtering" do
    it "can find a job by id" do
      scope = Postjob::Queue.search("postjobs", id: id)
      result = Simple::SQL.ask scope, into: Hash
      expect(result[:id]).to eq(id)
    end

    it "positive filters by tags" do
      scope = Postjob::Queue.search("postjobs", "initial" => "yay")
      result = Simple::SQL.all scope, into: OpenStruct
      expect(result.length).to eq(1)
      expect(result.first.id).to eq(id)
    end

    it "negative filters by tags" do
      scope = Postjob::Queue.search("postjobs", "initial" => "nay")
      result = Simple::SQL.all(scope)
      expect(result.length).to eq(0)
    end

    it "filter tags against empty array" do
      scope = Postjob::Queue.search("postjobs", "initial" => [])
      result = Simple::SQL.all(scope)
      expect(result.length).to eq(0)
    end

    it "filter tags against multiple matches" do
      scope = Postjob::Queue.search("postjobs", "initial" => [ "yay", "nay" ])
      result = Simple::SQL.all(scope)
      expect(result.length).to eq(1)
    end

    it "filter tags against a combination of multiple matches" do
      scope = Postjob::Queue.search("postjobs", "initial" => [ "yay", "nay" ], "multi" => "manyay")
      result = Simple::SQL.all(scope)
      expect(result.length).to eq(1)

      scope = Postjob::Queue.search("postjobs", "initial" => [ "yay", "nay" ], "multi" => "nay")
      result = Simple::SQL.all(scope)
      expect(result.length).to eq(0)

      scope = Postjob::Queue.search("postjobs", "initial" => [ "yay", "nay" ], "multi" => ["nay", "manyay"])
      result = Simple::SQL.all(scope)
      expect(result.length).to eq(1)
    end
  end
end
